/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <errno.h>
#include <assert.h>

#include "orfa_network.h"
#include "orfa_debug.h"


int orfa_FAKE_network_connect(char *server_name, struct connection *connection)
{
  return 0;
}


int orfa_FAKE_network_call(struct client_request *req)
{
  /* Check REQ type */
  LOG(3, "Sending type %d\n",
      (int)req->req.type);
  if ( req->req.type <= FIRST_REQ
       ||  req->req.type >= LAST_REQ )
    {
      ERROR("Bad request type %d !\n", (int)req->req.type);
      assert(0);
    }

  switch ( req->req.type )
    {
    case OPEN_REQ:
      {
	req->ans.return_value.int_value = DEFAULT_FD;
	req->ans.error = 0;
	break;
      }
    case OPENDIR_REQ:
      {
	req->ans.return_value.ptr_value = DEFAULT_DIR;
	req->ans.error = 0;
	break;
      }
    case CLOSE_REQ:
    case CLOSEDIR_REQ:
      {
	req->ans.return_value.int_value = 0;
	req->ans.error = 0;
	break;
      }
    case PREAD_REQ:
    case PWRITE_REQ:
      {
	req->ans.return_value.ssize_t_value = -1;
	req->ans.error = ENOSYS;
	break;
      }
    default:
      {
	req->ans.return_value.int_value = -1;
	req->ans.error = ENOSYS;
	break;
      }
    }

  return 0;
}
