/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFA_CLIENT_H__
#define __ORFA_CLIENT_H__


/* check if target type is well defined */
#if ( MAKE_WRAPPER + MAKE_LIB + MAKE_MOUNT != 1 )
#error "You must define one target type !"
#endif /* MAKE_WRAPPER or MAKE_LIB or MAKE_MOUNT */


/* globals */
#define CLIENT 1
#define _GNU_SOURCE 1

/* multithread safe case */
#if ( defined MULTITHREAD_SAFE_ENABLED )
#define _REENTRANT
#endif




/*********************
 * Basic definitions *
 *********************/

/* number of real FD for client
 * can be as big as you want, but really need ? */
#define FD_MAX 32

/* initial number of server */
#define DEFAULT_SERVER_MAX 10



/* general process information */
#include <sys/types.h>

extern struct context {
#if (defined MAKE_WRAPPER) || (defined MAKE_LIB)
  uid_t euid;
  gid_t egid;
  pid_t pid;
  mode_t umask;
  int concurrent_mode;
#endif
#ifdef NETWORK_BIP
  int data_send_tag;
  int big_data_send_tag;
#endif
} context;


/* debug */
#if CLIENT_DEBUG_LEVEL >= 0
#define INLINE /* nothing */
#else
#define INLINE inline
#endif


#if (defined MAKE_WRAPPER) || (defined MAKE_LIB)
/***********************************************
 * WRAPPER and LIB TARGET specific information *
 ***********************************************/

/* concurrent mode env var */
#define NON_CONCURRENT_MODE_ENV "ORFA_NON_CONCURRENT_MODE"

/* envs used during exec */
#define EXEC_PWD_ENV "ORFA_EXEC_PWD"
#define EXEC_PWD_ENV_LEN 13
#define EXEC_FD_ENV "ORFA_EXEC_FDLIST"
#define EXEC_FD_ENV_LEN 16
#define EXEC_DIR_ENV "ORFA_EXEC_DIRLIST"
#define EXEC_DIR_ENV_LEN 17
#define EXEC_SERVER_ENV "ORFA_EXEC_SERVERLIST"
#define EXEC_SERVER_ENV_LEN 20

/* assumed max len of fd converted to string
 * increae it if yu need more than 9999 decriptors */
#define FD_STRING_MAX_LEN 4

/* max len of connection_id converted to string (64bits) */
#define UINTMAX_STRING_MAX_LEN 20

/* path to orfa fs */
#define ORFA_PATH_PREFIX "/ORFA@"
#define ORFA_PATH_PREFIX_LENGTH 6

/* local file names to store data (unlinked after creation) */
/* file descriptor shared data */
#define ORFA_FD_FILE_PREFIX CLIENT_TEMP_FILE_PREFIX "/orfa_fd_"
/* dir decriptor shared data */
#define ORFA_DIR_DESCR_FILE_PREFIX CLIENT_TEMP_FILE_PREFIX "/orfa_dir_"
/* local copy of remote executable for exec */
#define ORFA_EXEC_FILE_PREFIX CLIENT_TEMP_FILE_PREFIX "/orfa_exec_"

/* when getting a new fd, we simply dup stderr */
#define FAKE_FD_TO_DUP 2

#ifdef DIR_CACHE_ENABLED
/* initial value and increasing factor of
 * number of dirents per readdir request */
#define DEFAULT_DIRENTS_CACHE_SIZE 16
#define DIRENTS_CACHE_INCREASING_FACTOR 2
#endif /* DIR_CACHE_ENABLED */


#elif (defined MAKE_MOUNT)
/*************************************
 * MOUNT TARGET specific information *
 *************************************/

/* initial value and increasing factor of
 * number of dirents per readdir request */
#ifdef DIR_CACHE_ENABLED
#define READDIR_MAX_DEFAULT 16
#define READDIR_MAX_INCREASING_FACTOR 12
#else
#define READDIR_MAX_DEFAULT 1
#define READDIR_MAX_INCREASING_FACTOR 1
#endif /* DIR_CACHE_ENABLED */


#endif /* MAKE_MOUNT */


/********************************
 * Network specific definitions *
 ********************************/

#define E_NETWORK_INIT_FAILED EIO
#define E_CONNECT_SERVER_FAILED ENOENT
#define E_REMOTE_CALL_FAILED EIO


#ifdef NETWORK_GM
/* GM definitions */

#define CLIENT_FIRST_PORT_ID 4
#define CLIENT_BOARD_NUM 0


#elif ( defined NETWORK_FAKE )
/* FAKE definitions */

#define DEFAULT_FD 17
#define DEFAULT_DIR 345678


#endif /* GM or BIP or FAKE network */


/************************************
 * general server request structure *
 ************************************/

#include "orfa_protocol.h"
#include "orfa_common_network.h"


struct client_request {
  struct request req;
  void *in_buf;
  
  struct answer ans;
  void *out_buf;
  size_t out_buf_len;

  struct connection *connection;

#ifdef NETWORK_GM
  void *real_in_buf;
  void *real_out_buf;
  int in_buf_size;
  int status;
#endif
#if NETWORK_BIP
  long answer_recv_id;
  long data_recv_id;
#endif
};


#endif /* __ORFA_CLIENT_H__ */
