/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFA_MISC_H__
#define __ORFA_MISC_H__


#include <sys/types.h>

#include "orfa_portable.h"


/**********
 * Atomic *
 **********/

#ifndef CONFIG_SMP
#define CONFIG_SMP 1
#endif
#include <asm/atomic.h>


/**************
 * Spin Locks *
 **************/

#include <asm/system.h>

typedef int spinlock_t;

static inline void spin_lock(spinlock_t *lock)
{
  while ( xchg(lock, 1) );
}

static inline void spin_unlock(spinlock_t *lock)
{
  *lock = 0;
}

#define SPIN_LOCK_INIT 0
#define spin_lock_init(s) spin_unlock(s)


/***********
 * Various *
 ***********/

int access_test(uid_t uid,
		gid_t gid,
		struct portable_stat*,
		int mode);


#endif /* __ORFA_MISC_H__ */
