/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFA_NETWORK_H__
#define __ORFA_NETWORK_H__


#include "orfa_protocol.h"
#include "orfa_client.h"
#include "orfa_server.h"


/* general calls really defined here */
extern int orfa_network_ready;

extern int orfa_network_init();
extern void orfa_network_finish();
extern int orfa_network_get_server(char *server_name);
extern int orfa_network_connect(char *server_name, struct connection *connection);
extern int orfa_network_call(int server, struct client_request *req);
#ifdef CONNECTION_ID_ENABLED
extern connection_id_t orfa_network_connection_call(int server,
						    connection_id_t id,
						    int op);
#endif

/* calls for fork */
extern void orfa_network_before_fork();
extern void orfa_network_after_fork_parent();
extern void orfa_network_after_fork_child();

/* calls for exec */
extern void orfa_network_before_exec();
extern void orfa_network_after_exec_failed();


/* network specific definitions */
#if ( defined NETWORK_TCP )
# include "orfa_TCP_network.h"
#elif ( defined NETWORK_UDP )
# include "orfa_UDP_network.h"
#elif ( defined NETWORK_GM )
# include "orfa_GM_network.h"
#elif ( defined NETWORK_BIP )
# include "orfa_BIP_network.h"
#elif ( defined NETWORK_FAKE )
# include "orfa_FAKE_network.h"
#else
# error no network defined
#endif /* TCP or UDP or GM or BIP or FAKE*/

/* regcache specific definitions */
#include "orfa_regcache.h"


#endif /* __ORFA_NETWORK_H__ */
