/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFA_PORTABLE_H__
#define __ORFA_PORTABLE_H__


#include <sys/stat.h>
#include <sys/statfs.h>
#include <dirent.h>
#include <stdint.h>

#include "orfa_common_portable.h"


#ifdef PORTABLE_ENABLED


extern void convert_stat_portable_to_64(struct stat64 *st,
					struct portable_stat *pst);
extern void convert_stat_portable_to_32(struct stat *st,
					struct portable_stat *pst);

extern void convert_statfs_portable_to_64(struct statfs64 *st,
					  struct portable_statfs *pst);
extern void convert_statfs_portable_to_32(struct statfs *st,
					  struct portable_statfs *pst);

extern void convert_dirent_portable_to_64(struct dirent64 *d,
					  struct portable_dirent *pd);
extern void convert_dirent_portable_to_32(struct dirent *d,
					  struct portable_dirent *pd);

extern orfa_oflags_t convert_oflags_to_portable(int oflags);

extern int convert_errno_from_portable(orfa_error_t perror);

#define PORTABLE_CALL64(func, buftype, arg, buf, res) \
do { \
  struct portable_##buftype pbuf; \
  res = orfa_##func##64(arg, &pbuf); \
  convert_##buftype##_portable_to_64(buf, &pbuf); \
} while ( 0 )

#define PORTABLE_CALL32(func, buftype, arg, buf, res) \
do { \
  struct portable_##buftype pbuf; \
  res = orfa_##func##64(arg, &pbuf); \
  convert_##buftype##_portable_to_32(buf, &pbuf); \
} while ( 0 )


#else /* PORTABLE_ENABLED */


#define convert_oflags_to_portable(x) (x)

#define convert_errno_from_portable(x) (x)

#define PORTABLE_CALL64(func, buftype, arg, buf, res) \
res = orfa_##func##64(arg, buf)

#define PORTABLE_CALL32(func, buftype, arg, buf, res) \
do {                                                         \
  struct buftype##64 buf64; \
  res = orfa_##func##64(arg, &buf64); \
  convert_##buftype##_64_to_32(buf, &buf64); \
} while ( 0 )


#endif /* PORTABLE_ENABLED */


#endif /* __ORFA_PORTABLE_H__ */
