/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFA_REAL_CALLS_H__
#define __ORFA_REAL_CALLS_H__


#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/statfs.h>
#include <dirent.h>
#include <utime.h>
#include <malloc.h>


/* include real_calls extern definitions */
#undef REAL_CALLS_EXTERN
#undef REAL_CALLS_STATIC
#undef REAL_CALLS_FIX
#define REAL_CALLS_EXTERN
#include "orfa_real_calls_list.h"


/* Fixing calls */
extern int real_calls_initialized;
extern void fix_all_real_calls();

#define CHECK_REAL_CALLS()       \
do {                             \
  if ( !real_calls_initialized ) \
    fix_all_real_calls();        \
} while ( 0 )


/* Forced lowest level real calls */
#ifdef MAKE_WRAPPER

#if HAVE___LIBC_OPEN
#define REAL_OPEN real___libc_open
#elif HAVE___OPEN
#define REAL_OPEN __open
#elif HAVE__OPEN
#define REAL_OPEN _open
#else
#define REAL_OPEN open
#endif

#if HAVE___LIBC_CLOSE
#define REAL_CLOSE real___libc_close
#elif HAVE___CLOSE
#define REAL_CLOSE real___close
#elif HAVE__CLOSE
#define REAL_CLOSE real__close
#else
#define REAL_CLOSE real_close
#endif

#if HAVE___LIBC_READ
#define REAL_READ real___libc_read
#elif HAVE___READ
#define REAL_READ real___read
#elif HAVE__READ
#define REAL_READ real__read
#else
#define REAL_READ real_read
#endif

#if HAVE___LIBC_WRITE
#define REAL_WRITE real___libc_write
#elif HAVE___WRITE
#define REAL_WRITE real___write
#elif HAVE__WRITE
#define REAL_WRITE real__write
#else
#define REAL_WRITE real_write
#endif

#if HAVE___MMAP
#define REAL_MMAP real___mmap
#elif HAVE__MMAP
#define REAL_MMAP real__mmap
#else
#define REAL_MMAP real_mmap
#endif

#if HAVE___MREMAP
#define REAL_MREMAP real___mremap
#elif HAVE__MREMAP
#define REAL_MREMAP real__mremap
#else
#define REAL_MREMAP real_mremap
#endif

#if HAVE___MUNMAP
#define REAL_MUNMAP real___munmap
#elif HAVE__MUNMAP
#define REAL_MUNMAP real__munmap
#else
#define REAL_MUNMAP real_munmap
#endif

#if HAVE___LIBC_FCNTL
#define REAL_FCNTL real___libc_fcntl
#elif HAVE___FCNTL
#define REAL_FCNTL real___fcntl
#elif HAVE__FCNTL
#define REAL_FCNTL real__fcntl
#else
#define REAL_FCNTL real_fcntl
#endif

#if HAVE___DUP
#define REAL_DUP real___dup
#elif HAVE__DUP
#define REAL_DUP real__dup
#else
#define REAL_DUP real_dup
#endif

#if HAVE___DUP2
#define REAL_DUP2 real___dup2
#elif HAVE__DUP2
#define REAL_DUP2 real__dup2
#else
#define REAL_DUP2 real_dup2
#endif

#define REAL_FTRUNCATE real_ftruncate
#define REAL_UNLINK real_unlink
#define REAL_GETCWD real_getcwd
#define REAL_CHDIR real_chdir
#define REAL_EXECVE real_execve
#define REAL_FEXECVE real_fexecve
#define REAL_UMASK real_umask

#elif (defined MAKE_LIB)

#define REAL_OPEN open
#define REAL_CLOSE close
#define REAL_READ read
#define REAL_WRITE write
#define REAL_MMAP mmap
#define REAL_MREMAP mremap
#define REAL_DUP2 dup2
#define REAL_MUNMAP munmap
#define REAL_FCNTL fcntl
#define REAL_DUP dup
#define REAL_FTRUNCATE ftruncate
#define REAL_UNLINK unlink
#define REAL_GETCWD getcwd
#define REAL_CHDIR chdir
#define REAL_EXECVE execve
#define REAL_FEXECVE fexecve
#define REAL_UMASK umask

#endif /* (defined MAKE_WRAPPER) || (defined MAKE_LIB) */


#endif /* __ORFA_REAL_CALLS_H__ */
