/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* This file is a little special.
 * It is used four times in a very different way.
 * Each REAL_CALL defines a line whose prototype vary
 * with the way this file is included. */

#undef REAL_CALL
#if (defined REAL_CALLS_EXTERN)
/* External definition in real_calls.h */
#define REAL_CALL(x,y,z) extern x (*real_##y) z
#elif (defined REAL_CALLS_STATIC)
/* Real definition in real_calls.c */
#define REAL_CALL(x,y,z) x (*real_##y) z
#elif (defined REAL_CALLS_FIX)
/* Fixing call in real_call.c */
#define REAL_CALL(x,y,z) FIX(y)
#else
#error Need to define how to use real calls list ?!
#endif


#include "orfa_symbols.h"


/* Open */
REAL_CALL(int, open, (const char*, int, ...));
#ifdef HAVE__OPEN
REAL_CALL(int, _open, (const char*, int, ...));
#elif HAVE___OPEN
REAL_CALL(int, __open, (const char*, int, ...));
#endif
#ifdef HAVE___LIBC_OPEN
REAL_CALL(int, __libc_open, (const char*, int, ...));
#endif
/* Open64 */
REAL_CALL(int, open64, (const char*, int, ...));
#ifdef HAVE__OPEN64
REAL_CALL(int, _open64, (const char*, int, ...));
#elif HAVE___OPEN64
REAL_CALL(int, __open64, (const char*, int, ...));
#endif
#ifdef HAVE___LIBC_OPEN64
REAL_CALL(int, __libc_open64, (const char*, int, ...));
#endif
/* creat/creat64 is bound to open */

/* close */
REAL_CALL(int, close, (int fd));
#ifdef HAVE__CLOSE
REAL_CALL(int, _close, (int fd));
#elif HAVE___CLOSE
REAL_CALL(int, __close, (int fd));
#endif
#ifdef HAVE___LIBC_CLOSE
REAL_CALL(int, __libc_close, (int fd));
#endif

/* read */
REAL_CALL(ssize_t, read, (int fd, void *buf, size_t count));
#ifdef HAVE__READ
REAL_CALL(ssize_t, _read, (int fd, void *buf, size_t count));
#elif HAVE___READ
REAL_CALL(ssize_t, __read, (int fd, void *buf, size_t count));
#endif
#ifdef HAVE___LIBC_READ
REAL_CALL(ssize_t, __libc_read, (int fd, void *buf, size_t count));
#endif
/* pread */
REAL_CALL(ssize_t, pread, (int fd, void *buf, size_t count, off_t offset));
#ifdef HAVE__PREAD
REAL_CALL(ssize_t, _pread, (int fd, void *buf, size_t count, off_t offset));
#elif HAVE___PREAD
REAL_CALL(ssize_t, __pread, (int fd, void *buf, size_t count, off_t offset));
#endif
#ifdef HAVE___LIBC_PREAD
REAL_CALL(ssize_t, __libc_pread, (int fd, void *buf, size_t count, off_t offset));
#endif
/* pread64 */
REAL_CALL(ssize_t, pread64, (int fd, void *buf, size_t count, off64_t offset));
#ifdef HAVE__PREAD64
REAL_CALL(ssize_t, _pread64, (int fd, void *buf, size_t count, off64_t offset));
#elif HAVE___PREAD64
REAL_CALL(ssize_t, __pread64, (int fd, void *buf, size_t count, off64_t offset));
#endif
#ifdef HAVE___LIBC_PREAD64
REAL_CALL(ssize_t, __libc_pread64, (int fd, void *buf, size_t count, off64_t offset));
#endif

/* write */
REAL_CALL(ssize_t, write, (int fd, const void *buf, size_t count));
#ifdef HAVE__WRITE
REAL_CALL(ssize_t, _write, (int fd, const void *buf, size_t count));
#elif HAVE___WRITE
REAL_CALL(ssize_t, __write, (int fd, const void *buf, size_t count));
#endif
#ifdef HAVE___LIBC_WRITE
REAL_CALL(ssize_t, __libc_write, (int fd, const void *buf, size_t count));
#endif
/* pwrite */
REAL_CALL(ssize_t, pwrite, (int fd, const void *buf, size_t count, off_t offset));
#ifdef HAVE__PWRITE
REAL_CALL(ssize_t, _pwrite, (int fd, const void *buf, size_t count, off_t offset));
#elif HAVE___PWRITE
REAL_CALL(ssize_t, __pwrite, (int fd, const void *buf, size_t count, off_t offset));
#endif
#ifdef HAVE___LIBC_PWRITE
REAL_CALL(ssize_t, __libc_pwrite, (int fd, const void *buf, size_t count, off_t offset));
#endif
/* pwrite64 */
REAL_CALL(ssize_t, pwrite64, (int fd, const void *buf, size_t count, off64_t offset));
#ifdef HAVE__PWRITE64
REAL_CALL(ssize_t, _pwrite64, (int fd, const void *buf, size_t count, off64_t offset));
#elif HAVE___PWRITE64
REAL_CALL(ssize_t, __pwrite64, (int fd, const void *buf, size_t count, off64_t offset));
#endif
#ifdef HAVE___LIBC_PWRITE64
REAL_CALL(ssize_t, __libc_pwrite64, (int fd, const void *buf, size_t count, off64_t offset));
#endif

/* sendfile */
#ifdef HAVE_SENDFILE
REAL_CALL(ssize_t, sendfile, (int out_fd, int in_fd, off_t *offset, size_t count));
#endif
#ifdef HAVE_SENDFILE64
REAL_CALL(ssize_t, sendfile64, (int out_fd, int in_fd, off64_t *offset, size_t count));
#endif

/* lseek */
REAL_CALL(off_t, lseek, (int, off_t, int));
#ifdef HAVE__LSEEK
REAL_CALL(off_t, _lseek, (int, off_t, int));
#elif HAVE___LSEEK
REAL_CALL(off_t, __lseek, (int, off_t, int));
#endif
#ifdef HAVE___LIBC_LSEEK
REAL_CALL(off_t, __libc_lseek, (int, off_t, int));
#endif
/* lseek64 */
REAL_CALL(off64_t, lseek64, (int, off64_t, int));
#ifdef HAVE__LSEEK64
REAL_CALL(off64_t, _lseek64, (int, off64_t, int));
#elif HAVE___LSEEK64
REAL_CALL(off64_t, __lseek64, (int, off64_t, int));
#endif
#ifdef HAVE___LIBC_LSEEK64
REAL_CALL(off64_t, __libc_lseek64, (int, off64_t, int));
#endif
/* llseek */
#ifdef HAVE_LLSEEK
REAL_CALL(loff_t, llseek, (int fd, loff_t ofs, int whence));
#endif
/* _IO_file_seek */
#ifdef HAVE__IO_FILE_SEEK
REAL_CALL(_IO_off64_t, _IO_file_seek, (struct _IO_FILE*, _IO_off64_t, int));
#endif

/* truncate */
REAL_CALL(int, truncate, (const char*, off_t));
#ifdef HAVE_TRUNCATE64
REAL_CALL(int, truncate64, (const char*, off64_t));
#endif
REAL_CALL(int, ftruncate, (int, off_t));
#ifdef HAVE_FTRUNCATE64
REAL_CALL(int, ftruncate64, (int, off64_t));
#endif

/* fstat */
#ifdef HAVE___FXSTAT
REAL_CALL(int, __fxstat, (int, int, struct stat*));
#else
REAL_CALL(int, fstat, (int, struct stat*));
#endif
#ifdef HAVE__FSTAT
REAL_CALL(int, _fstat, (int, struct stat*));
#elif HAVE___FSTAT
REAL_CALL(int, __fstat, (int, struct stat*));
#endif
/* stat */
#ifdef HAVE___XSTAT
REAL_CALL(int, __xstat, (int, const char*, struct stat*));
#else
REAL_CALL(int, stat, (const char*, struct stat*));
#endif
#ifdef HAVE__STAT
REAL_CALL(int, _stat, (const char*, struct stat*));
#elif HAVE___STAT
REAL_CALL(int, __stat, (const char*, struct stat*));
#endif
/* lstat */
#ifdef HAVE___LXSTAT
REAL_CALL(int, __lxstat, (int, const char*, struct stat*));
#else
REAL_CALL(int, lstat, (const char*, struct stat*));
#endif
#ifdef HAVE__LSTAT
REAL_CALL(int, _lstat, (const char*, struct stat*));
#elif HAVE___LSTAT
REAL_CALL(int, __lstat, (const char*, struct stat*));
#endif
/* fstat64 */
#ifdef HAVE___FXSTAT64
REAL_CALL(int, __fxstat64, (int, int, struct stat64*));
#else
REAL_CALL(int, fstat64, (int, struct stat64*));
#endif
#ifdef HAVE__FSTAT64
REAL_CALL(int, _fstat64, (int, struct stat64*));
#elif HAVE___FSTAT64
REAL_CALL(int, __fstat64, (int, struct stat64*));
#endif
/* stat64 */
#ifdef HAVE___XSTAT64
REAL_CALL(int, __xstat64, (int, const char*, struct stat64*));
#else
REAL_CALL(int, stat64, (const char*, struct stat64*));
#endif
#ifdef HAVE__STAT64
REAL_CALL(int, _stat64, (const char*, struct stat64*));
#elif HAVE___STAT64
REAL_CALL(int, __stat64, (const char*, struct stat64*));
#endif
/* lstat64 */
#ifdef HAVE___LXSTAT64
REAL_CALL(int, __lxstat64, (int, const char*, struct stat64*));
#else
REAL_CALL(int, lstat64, (const char*, struct stat64*));
#endif
#ifdef HAVE__LSTAT64
REAL_CALL(int, _lstat64, (const char*, struct stat64*));
#elif HAVE___LSTAT64
REAL_CALL(int, __lstat64, (const char*, struct stat64*));
#endif

/* statfs */
REAL_CALL(int, statfs, (const char*, struct statfs*));
#ifdef HAVE_STATFS64
REAL_CALL(int, statfs64, (const char*, struct statfs64*));
#endif
REAL_CALL(int, fstatfs, (int, struct statfs*));
#ifdef HAVE_FSTATFS64
REAL_CALL(int, fstatfs64, (int, struct statfs64*));
#endif

/* mknod */
#ifdef HAVE___XMKNOD
REAL_CALL(int, __xmknod, (int, const char*, mode_t, dev_t*));
#else
REAL_CALL(int, mknod, (const char*, mode_t, dev_t));
#endif
#ifdef HAVE__MKNOD
REAL_CALL(int, _mknod, (const char*, mode_t, dev_t));
#elif HAVE___MKNOD
REAL_CALL(int, __mknod, (const char*, mode_t, dev_t));
#endif

/* tree things */
REAL_CALL(int, mkdir, (const char*, mode_t));
REAL_CALL(int, rmdir, (const char*));
REAL_CALL(int, unlink, (const char*));
REAL_CALL(int, rename, (const char*, const char*));
REAL_CALL(int, link, (const char*, const char*));
REAL_CALL(int, symlink, (const char*, const char*));
REAL_CALL(int, readlink, (const char*, char*, size_t));

/* access mode */
REAL_CALL(int, fchmod, (int, mode_t));
REAL_CALL(int, chmod, (const char*, mode_t));
REAL_CALL(int, access, (const char*, int));
REAL_CALL(int, euidaccess, (const char*, int));

/* owner */
REAL_CALL(int, fchown, (int, uid_t, gid_t));
REAL_CALL(int, chown, (const char*, uid_t, gid_t));
REAL_CALL(int, lchown, (const char*, uid_t, gid_t));

/* fcntl */
REAL_CALL(int, fcntl, (int, int, long));
#ifdef HAVE__FCNTL
REAL_CALL(int, _fcntl, (int, int, long));
#elif HAVE___FCNTL
REAL_CALL(int, __fcntl, (int, int, long));
#endif
#ifdef HAVE___LIBC_FCNTL
REAL_CALL(int, __libc_fcntl, (int, int, long));
#endif

/* flock */
REAL_CALL(int, flock, (int, int));

/* cwd */
REAL_CALL(int, fchdir, (int));
REAL_CALL(int, chdir, (const char*));
REAL_CALL(char *, getcwd, (char*, size_t));

/* DIR */
REAL_CALL(DIR *, opendir, (const char*));
REAL_CALL(int, closedir, (DIR*));
REAL_CALL(struct dirent *, readdir, (DIR*));
#ifdef HAVE_READDIR64
REAL_CALL(struct dirent64 *, readdir64, (DIR*));
#endif
#ifdef HAVE_READDIR_R
REAL_CALL(int, readdir_r, (DIR*, struct dirent*, struct dirent**));
#endif
#ifdef HAVE_READDIR64_R
REAL_CALL(int, readdir64_r, (DIR*, struct dirent64*, struct dirent64**));
#endif
REAL_CALL(void, seekdir, (DIR*, off_t));
REAL_CALL(off_t, telldir, (DIR*));
REAL_CALL(void, rewinddir, (DIR*));

/* utime */
REAL_CALL(int, utime, (const char*, const struct utimbuf*));
REAL_CALL(int, utimes, (const char*, const struct timeval*));

/* sync */
REAL_CALL(int, fsync, (int));
#ifdef HAVE__FSYNC
REAL_CALL(int, _fsync, (int));
#elif HAVE___FSYNC
REAL_CALL(int, __fsync, (int));
#endif
#ifdef HAVE___LIBC_FSYNC
REAL_CALL(int, __libc_fsync, (int));
#endif
#ifdef HAVE_FDATASYNC
REAL_CALL(int, fdatasync, (int));
#endif

/* dup */
REAL_CALL(int, dup, (int));
#ifdef HAVE__DUP
REAL_CALL(int, _dup, (int));
#elif HAVE___DUP
REAL_CALL(int, __dup, (int));
#endif
/* dup2 */
REAL_CALL(int, dup2, (int, int));
#ifdef HAVE__DUP2
REAL_CALL(int, _dup2, (int, int));
#elif HAVE___DUP2
REAL_CALL(int, __dup2, (int, int));
#endif

/* mmap */
REAL_CALL(caddr_t, mmap, (void*, size_t, int, int, int, off_t));
#ifdef HAVE_MMAP64
REAL_CALL(caddr_t, mmap64, (void*, size_t, int, int, int, off64_t));
#endif
REAL_CALL(void *, mremap, (void*, size_t, size_t, unsigned long));
REAL_CALL(int, munmap, (void*, size_t));
/* msync */
REAL_CALL(int, msync, (const void*, size_t, int));
#ifdef HAVE___MSYNC
REAL_CALL(int, _msync, (const void*, size_t, int));
#elif HAVE___MSYNC
REAL_CALL(int, __msync, (const void*, size_t, int));
#endif
#ifdef HAVE___LIBC_MSYNC
REAL_CALL(int, __libc_msync, (const void*, size_t, int));
#endif

/* exec */
REAL_CALL(int, execve, (const char*, char *const[], char *const[]));
REAL_CALL(int, fexecve, (int, char *const[], char *const[]));
REAL_CALL(int, execv, (const char*, char *const[]));
REAL_CALL(int, execle, (const char*, const char*, ...));
REAL_CALL(int, execl, (const char*, const char*, ...));

/* fork */
REAL_CALL(pid_t, fork, ());
#ifdef HAVE__FORK
REAL_CALL(pid_t, _fork, ());
#elif HAVE___FORK
REAL_CALL(pid_t, __fork, ());
#endif
#ifdef HAVE___LIBC_FORK
REAL_CALL(pid_t, __libc_fork, ());
#endif

/* clone */
#if MULTITHREAD_SAFE_ENABLED
REAL_CALL(int , __clone, (int(*)(void*), void*, int, void *));
#endif

/* euid/egid/umask needed from extra gestion */
REAL_CALL(int, setuid, (uid_t));
REAL_CALL(int, seteuid, (uid_t));
REAL_CALL(int, setreuid, (uid_t, uid_t));
REAL_CALL(int, setresuid, (uid_t, uid_t, uid_t));
REAL_CALL(int, setgid, (gid_t));
REAL_CALL(int, setegid, (gid_t));
REAL_CALL(int, setregid, (gid_t, gid_t));
REAL_CALL(int, setresgid, (gid_t, gid_t, gid_t));
REAL_CALL(mode_t, umask, (mode_t));

/* getdents is not really supported but supported for LOCAL calls */
#ifdef HAVE_GETDENTS
REAL_CALL(int, getdents, (int, void*, int));
#endif
#ifdef HAVE_GETDENTS64
REAL_CALL(int, getdents64, (int, void*, int));
#endif

/* ACL */
#ifdef HAVE_LIBACL
#include <sys/acl.h>
REAL_CALL(acl_t, acl_get_fd, (int));
REAL_CALL(int, acl_set_fd, (int, acl_t));
REAL_CALL(int, acl_extended_fd, (int));
REAL_CALL(acl_t, acl_get_file, (const char*, acl_type_t));
REAL_CALL(int, acl_set_file, (const char*, acl_type_t, acl_t));
REAL_CALL(int, acl_extended_file, (const char*));
REAL_CALL(int, acl_delete_def_file, (const char*));
#endif
