/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <sys/types.h>
#include <unistd.h>

#include "orfa_portable.h"


int access_test(uid_t uid,
		gid_t gid,
		struct portable_stat *st,
		int mode)
{
  /* This code was inspired
   * from glibc's implementation of euidaccess */
  
  int granted;

  /* Clear any bogus bits. */
  mode &= (X_OK | W_OK | R_OK);
#if R_OK != S_IROTH || W_OK != S_IWOTH || X_OK != S_IXOTH || F_OK != 0
# error Oops, portability assumptions incorrect.
#endif
  
  /* file exists ! */
  if (mode == F_OK)
    return 0;

  /* The super-user can read and write any file, and execute any file
     that anyone can execute. */
  if (uid == 0 && 
      ((mode & X_OK) == 0
       || (st->st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))))
    return 0;

  /* effective user granted ? */
  if (uid == st->st_uid)
    granted = (unsigned) (st->st_mode & (mode << 6)) >> 6;

  /* effective group granted ? */
  else if (gid == st->st_gid || group_member (st->st_gid))
    granted = (unsigned) (st->st_mode & (mode << 3)) >> 3;

  /* other granted ? */
  else
    granted = (st->st_mode & mode);

  /* final test */
  if (granted == mode)
    return 0;
  
  return -1;
}
