/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __COMMON_PORTABLE_H__
#define __COMMON_PORTABLE_H__


#ifdef PORTABLE_ENABLED


#include <stdint.h>

struct portable_stat {
  uint64_t st_dev;
  uint64_t st_ino;
  uint64_t st_mode;
  uint64_t st_nlink;
  uint64_t st_uid;
  uint64_t st_gid;
  uint64_t st_rdev;
  uint64_t st_size;
  uint64_t st_blksize;
  uint64_t st_blocks;
/* these fields must have different names
 * to avoid glibc header #define
 */
  uint64_t orfa_st_atime;
  uint64_t orfa_st_mtime;
  uint64_t orfa_st_ctime;
};

struct portable_statfs {
  uint64_t f_type;
  uint64_t f_bsize;
  uint64_t f_blocks;
  uint64_t f_bfree;
  uint64_t f_bavail;
  uint64_t f_files;
  uint64_t f_ffree;
  struct _f_fsid {
    uint64_t __val[2];
  } f_fsid;
  uint64_t f_namelen;
  uint64_t f_spare[6];
};

struct portable_dirent {
  uint64_t d_ino;
  uint64_t d_off;
  uint64_t d_reclen;
  uint64_t d_type;
  char d_name[256];
};

#define P_O_RDONLY 0x00000
#define P_O_WRONLY 0x00001
#define P_O_RDWR 0x00002
#define P_O_ACCMODE 0x00003

#define P_O_CREAT 0x00010
#define P_O_EXCL 0x00020
#define P_O_NOCTTY 0x00040
#define P_O_TRUNC 0x00080
#define P_O_APPEND 0x00100
#define P_O_NONBLOCK 0x00200
#define P_O_NDELAY 0x00400
#define P_O_SYNC 0x00800
#define P_O_FSYNC 0x01000
#define P_O_ASYNC 0x02000
#define P_O_DSYNC 0x04000
#define P_O_RSYNC 0x08000
#define P_O_NOFOLLOW 0x10000
#define P_O_DIRECTORY 0x20000
#define P_O_LARGEFILE 0x40000
#define P_O_DIRECT 0x80000


#else /* PORTABLE_ENABLED */


#include <fcntl.h>
#include <sys/stat.h>
#include <sys/statfs.h>

#define portable_stat stat64
#define portable_statfs statfs64
#define portable_dirent dirent64
#define portable_oflags uint64

#define P_O_RDONLY O_RDONLY
#define P_O_WRONLY O_WRONLY
#define P_O_RDWR O_RDWR
#define P_O_ACCMODE O_ACCMODE

#define P_O_CREAT O_CREAT
#define P_O_EXCL O_EXCL
#define P_O_NOCTTY O_NOCTTY
#define P_O_TRUNC O_TRUNC
#define P_O_APPEND O_APPEND
#define P_O_NONBLOCK O_NONBLOCK
#define P_O_NDELAY O_NDELAY
#define P_O_SYNC O_SYNC
#define P_O_FSYNC O_FSYNC
#define P_O_ASYNC O_ASYNC
#define P_O_DSYNC O_DSYNC
#define P_O_RSYNC O_RSYNC
#define P_O_NOFOLLOW O_NONFOLLOW
#define P_O_DIRECTORY O_DIRECTORY
#define P_O_LARGEFILE O_LARGEFILE
#define P_O_DIRECT O_DIRECT


#endif /* PORTABLE_ENABLED */


#endif /* __COMMON_PORTABLE_H__ */
