/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFAS_BIP_NETWORK_H__
#define __ORFAS_BIP_NETWORK_H__


extern int BIP_network_init();
extern void BIP_network_finish();
extern int BIP_network_send(struct server_request *req);

#ifdef AIO_MIXED_WITH_OTHER_ASYNC_API
extern void BIP_network_test();
#else
extern void BIP_network_wait();
#endif


/* This allow functions to wait for send token
 * before ti-sending. We assume this will never
 * be used in multi-sending-thread case */
extern long bip_send_token;
extern void (*bip_send_callback)(void *data);
extern void *bip_send_callback_data;

#define BIP_WAIT_FOR_SEND_TOKEN()                   \
do {                                                \
if ( bip_send_token ) {                             \
  bip_swait(bip_send_token);                        \
  if ( bip_send_callback ) {                        \
    LOG(6, "Calling BIP send callback %p on %p\n",  \
	bip_send_callback, bip_send_callback_data); \
    bip_send_callback(bip_send_callback_data);      \
  }                                                 \
  bip_send_token = 0;                               \
} } while ( 0 )

#define BIP_TAKE_SEND_TOKEN(x) bip_send_token = x

#define BIP_FIX_SEND_CALLBACK(x,y) \
do {                               \
  bip_send_callback = x;           \
  bip_send_callback_data = y;      \
} while ( 0 )


#endif /* __ORFAS_BIP_NETWORK_H__ */
