/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFAS_RAM_FS_H__
#define __ORFAS_RAM_FS_H__


#include <utime.h>
#include <stdint.h>


#include "orfas_io.h"
#include "orfas_network.h"


extern int RAM_fs_init();
extern void RAM_fs_finish();

extern int RAM_fs_open(const char*, orfa_oflags_t, orfa_mode_t, void**);
extern int RAM_fs_close(void*);
extern orfa_soff_t RAM_fs_pread(void*, struct io_vector**, orfa_off_t, orfa_off_t);
extern orfa_soff_t RAM_fs_pwrite(void*, const char*, orfa_off_t, orfa_off_t);
extern int RAM_fs_ftruncate(void*, orfa_off_t);
extern int RAM_fs_truncate(const char*, orfa_off_t);
extern int RAM_fs_fstat(void*, struct portable_stat*);
extern int RAM_fs_stat(const char*, struct portable_stat*);
extern int RAM_fs_lstat(const char*, struct portable_stat*);
extern int RAM_fs_statfs(struct portable_statfs*);
extern int RAM_fs_mknod(const char*, orfa_mode_t, orfa_dev_t);
extern int RAM_fs_mkdir(const char*, orfa_mode_t);
extern int RAM_fs_rmdir(const char*);
extern int RAM_fs_unlink(const char*);
extern int RAM_fs_access(const char*, orfa_access_mode_t);
extern int RAM_fs_fchmod(void*, orfa_mode_t);
extern int RAM_fs_chmod(const char*, orfa_mode_t);
extern int RAM_fs_fchown(void*, orfa_uid_t, orfa_gid_t);
extern int RAM_fs_chown(const char*, orfa_uid_t, orfa_gid_t);
extern int RAM_fs_lchown(const char*, orfa_uid_t, orfa_gid_t);
extern int RAM_fs_rename(const char*, const char*);
extern int RAM_fs_link(const char*, const char*);
extern int RAM_fs_symlink(const char*, const char*);
extern int RAM_fs_readlink(const char*, char*, orfa_off_t);
extern int RAM_fs_fsync(void*);
extern int RAM_fs_fdatasync(void*);
extern int RAM_fs_utime(const char*, orfa_time_t, orfa_time_t, orfa_bool_t);
extern int RAM_fs_flock(void*, orfa_flock_op_t);
extern int RAM_fs_fcntl_setfl(void*, orfa_oflags_t);
extern int RAM_fs_opendir(const char*, void**);
extern int RAM_fs_preaddir(void*, orfa_off_t, struct io_vector**, orfa_off_t);
extern int RAM_fs_rewinddir(void*);
extern int RAM_fs_closedir(void*);

#ifdef CONNECTION_ID_ENABLED
extern void RAM_close_file_for_connection_id(struct connection_id*);
extern void RAM_set_fd_connection_id(void*, struct connection_id*);
extern void RAM_set_dir_connection_id(void*, struct connection_id*);
#endif /* ~CONNECTION_ID_ENABLED */


#endif /* __ORFAS_RAM_FS_H__ */
