/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFAS_DEBUG_H__
#define __ORFAS_DEBUG_H__


#include <stdio.h>


#define LOG_FD 2
#define LOG_FILE stderr



/********************************
 * MT-safe specific definitions *
 *********************************/

#if (defined QUEUE_THREAD_ARCH) || (defined CONNECTION_THREAD_ARCH)

/* defining the LOG mutex */
#include <pthread.h>
extern pthread_mutex_t LOG_mutex;
#define INIT_LOGS() pthread_mutex_t LOG_mutex = PTHREAD_MUTEX_INITIALIZER

#else

#define INIT_LOGS() /* nothing */

#endif



/**********
 * Errors *
 **********/

#if (defined QUEUE_THREAD_ARCH) || (defined CONNECTION_THREAD_ARCH)

#define ERROR(args...)                 \
do {                                   \
pthread_mutex_lock(&LOG_mutex);        \
fprintf(LOG_FILE, "ORFA server: " args); \
pthread_mutex_unlock(&LOG_mutex);      \
} while ( 0 )

#else

#define ERROR(args...) fprintf(LOG_FILE, "ORFA server: " args)

#endif



/*********
 * Debug *
 *********/

#if (defined QUEUE_THREAD_ARCH) || (defined CONNECTION_THREAD_ARCH)

#define LOG(level, args...)                  \
if ( level < SERVER_DEBUG_LEVEL ) {          \
int debug_int;                               \
pthread_mutex_lock(&LOG_mutex);              \
for(debug_int=1;debug_int<level;debug_int++) \
fprintf(LOG_FILE, "..");                     \
fprintf(LOG_FILE, args);                     \
pthread_mutex_unlock(&LOG_mutex);            \
}

#else

#define LOG(level, args...)                  \
if ( level < SERVER_DEBUG_LEVEL ) {          \
int debug_int;                               \
for(debug_int=1;debug_int<level;debug_int++) \
fprintf(LOG_FILE, "..");                     \
fprintf(LOG_FILE, args);                     \
}

#endif



/*************
 * Asserting *
 *************/

#if ( SERVER_DEBUG_LEVEL < 0 )

#define DEBUG_ASSERT(x) do { /* nothing */ } while ( 0 )

#else /* SERVER_DEBUG_LEVEL >= 0 */

#include <assert.h>
#define DEBUG_ASSERT(x) assert(x)

#endif /* SERVER_DEBUG_LEVEL >= 0 */


/************
 * Debuging *
 ************/

#if ( SERVER_DEBUG_LEVEL < 0 )

#define DEBUG_INSTR(x) /* nothing */

#else /* SERVER_DEBUG_LEVEL >= 0 */

#define DEBUG_INSTR(x) x

#endif /* SERVER_DEBUG_LEVEL >= 0 */


#endif /* __ORFAS_DEBUG_H__ */
