/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFAS_IO_H__
#define __ORFAS_IO_H__


#include <sys/uio.h>


struct io_vector {
  int nr;
  int do_not_free;
  struct iovec buf[0];
};

#define VECT_NR(vect) ((vect)->nr)
#define VECT_BUF(vect, i) (&((vect)->buf[i]))
#define BUF_PTR(buf) ((buf)->iov_base)
#define BUF_SIZE(buf) ((buf)->iov_len)
#define VECT_PTR(vect, i) ((vect)->buf[i].iov_base)
#define VECT_SIZE(vect, i) ((vect)->buf[i].iov_len)


extern struct io_vector *get_io_vector(int nr,
				       int do_not_free);
extern struct io_vector *get_io_monovector(void *ptr,
					   size_t size,
					   int do_not_free);
extern struct io_vector *get_io_monovector_alloc(size_t size);

extern void put_io_vector(struct io_vector *vect);

extern void resize_io_monovector(struct io_vector *vect,
				 size_t size);

extern struct io_vector *merge_io_vector(struct io_vector *vect);



#if ( defined STD_IO )

# define process_init STD_process_init 
# define process_finish STD_process_finish 
# define process_request STD_process_request
# define close_file_for_connection_id STD_close_file_for_connection_id
# include "orfas_STD_io.h"

#elif ( defined AIO_IO )

/* AIO is based on STD */
# define process_init STD_process_init
# define process_finish STD_process_finish
# define process_request AIO_process_request
# define close_file_for_connection_id STD_close_file_for_connection_id
# include "orfas_AIO_io.h"
# include "orfas_STD_io.h"

#elif ( defined FAKE_IO )

# define process_init FAKE_process_init
# define process_finish FAKE_process_finish
# define process_request FAKE_process_request
# define close_file_for_connection_id(x) /* nothing */
# include "orfas_FAKE_io.h"

#elif ( defined RAM_IO )

# define process_init RAM_process_init
# define process_finish RAM_process_finish
# define process_request RAM_process_request
# define close_file_for_connection_id RAM_close_file_for_connection_id
# include "orfas_RAM_io.h"
# include "orfas_RAM_fs.h"

#endif /* STD_IO or AIO_IO or FAKE_IO or RAM_IO */


#endif /* __ORFAS_IO_H__ */
