/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFAS_NETWORK_H__
#define __ORFAS_NETWORK_H__



#ifdef CONNECTION_ID_ENABLED
extern void process_connection_id(struct server_request *req,
				  struct connection_id **idptr);
extern void put_connection_id(struct connection_id *id);
#endif /* CONNECTION_ID_ENABLED */



/*******
 * TCP *
 *******/

#if ( defined NETWORK_TCP )

#define network_init TCP_network_init
#define network_finish TCP_network_finish
#define network_send TCP_network_send

#define network_test TCP_network_test
#define network_wait TCP_network_wait

#define network_buffer_malloc malloc
#define network_buffer_free free

#include "orfas_TCP_network.h"


/*******
 * UDP *
 *******/

#elif ( defined NETWORK_UDP )

#define network_init UDP_network_init
#define network_finish UDP_network_finish
#define network_send UDP_network_send

#define network_test UDP_network_test
#define network_wait UDP_network_wait

#define network_buffer_malloc malloc
#define network_buffer_free free

#include "orfas_UDP_network.h"


/******
 * GM *
 ******/

#elif ( defined NETWORK_GM )

#define network_init GM_network_init
#define network_finish GM_network_finish
#define network_send GM_network_send

#define network_test GM_network_test
#define network_wait GM_network_wait

#define network_buffer_malloc GM_buffer_malloc
#define network_buffer_free GM_buffer_free

#include "orfas_GM_network.h"


/*******
 * BIP *
 *******/

#elif ( defined NETWORK_BIP )

#define network_init BIP_network_init
#define network_finish BIP_network_finish
#define network_send BIP_network_send

#define network_test BIP_network_test
#define network_wait BIP_network_wait

#include <malloc.h>
#define network_buffer_malloc(x) memalign(WORD_SIZE, (x+WORD_SIZE-1)&~(WORD_SIZE-1))
#define network_buffer_free free

#include "orfas_BIP_network.h"


#endif /* TCP or UDP or GM or BIP */


#endif /* __ORFAS_NETWORK_H__ */
