/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFAS_PORTABLE_H__
#define __ORFAS_PORTABLE_H__


#include <sys/stat.h>
#include <sys/statfs.h>
#include <dirent.h>
#include <stdint.h>

#include "orfa_common_portable.h"


#ifdef PORTABLE_ENABLED

extern void convert_stat_64_to_portable(struct portable_stat *pst,
					struct stat64 *st);

extern void convert_statfs_64_to_portable(struct portable_statfs *pst,
					  struct statfs64 *st);

extern void convert_dirent_64_to_portable(struct portable_dirent *pd,
					  struct dirent64 *d);

extern int convert_oflags_from_portable(orfa_oflags_t poflags);

extern orfa_error_t convert_errno_to_portable(int error);

#define convert_stat_time_to_portable(st, type, value) st->orfa_st_##type = value


#else /* PORTABLE_ENABLED */


#define convert_oflags_from_portable(x) (x)

#define convert_errno_to_portable(x) (x)

#define convert_stat_time_to_portable(st, type, value) st->st_##type = value


#endif /* PORTABLE_ENABLED */


#endif /* __ORFAS_PORTABLE_H__ */
