/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdlib.h>
#include <string.h>

#include "orfas_io.h"
#include "orfas_network.h"
#include "orfas_debug.h"



struct io_vector *get_io_vector(int nr,
				int do_not_free)
{
  struct io_vector *vect = (struct io_vector*)
    malloc(sizeof(struct io_vector)+nr*sizeof(struct iovec));

  vect->nr = nr;
  vect->do_not_free = do_not_free;

  return vect;
}


struct io_vector *get_io_monovector(void *ptr,
				    size_t size,
				    int do_not_free)
{
  struct io_vector *vect = get_io_vector(1, do_not_free);

  vect->buf[0].iov_base = ptr;
  vect->buf[0].iov_len = size;

  return vect;
}


struct io_vector *get_io_monovector_alloc(size_t size)
{
  return get_io_monovector(network_buffer_malloc(size),
			   size,
			   0);
}


void put_io_vector(struct io_vector *vect)
{
  if ( vect ) {
    if ( !vect->do_not_free ) {
      int i;
      for(i=0; i<vect->nr; i++)
	network_buffer_free(vect->buf[i].iov_base);
    }
    free(vect);
  }
}


void resize_io_monovector(struct io_vector *vect, size_t size)
{
  DEBUG_ASSERT( size<=vect->buf[0].iov_len );
  vect->buf[0].iov_len = size;
}


struct io_vector *merge_io_vector(struct io_vector *src)
{
  size_t size = 0;
  struct io_vector *new;
  void *current_ptr;
  int i;

  if ( src == NULL )
    return NULL;

  if ( src->nr == 1 )
    return src;

  for(i=0; i<src->nr; i++)
    size += src->buf[i].iov_len;
  
  new = get_io_monovector_alloc(size);
  
  current_ptr = new->buf[0].iov_base;

  for(i=0; i<src->nr; i++) {
    int to_copy = src->buf[i].iov_len;
    memcpy(current_ptr, src->buf[i].iov_base, to_copy);
    current_ptr += to_copy;
  }

  put_io_vector(src);

  return new;
}
