/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2002 Brice Goglin <Brice.Goglin@ens-lyon.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <string.h>
#include <fcntl.h>

#include "orfas_portable.h"


void convert_stat_64_to_portable(struct portable_stat *pst,
				 struct stat64 *st)
{
  pst->st_dev = st->st_dev;
  pst->st_ino = st->st_ino;
  pst->st_mode = st->st_mode;
  pst->st_nlink = st->st_nlink;
  pst->st_uid = st->st_uid;
  pst->st_gid = st->st_gid;
  pst->st_rdev = st->st_rdev;
  pst->st_size = st->st_size;
  pst->st_blksize = st->st_blksize;
  pst->st_blocks = st->st_blocks;
  pst->orfa_st_atime = st->st_atime;
  pst->orfa_st_mtime = st->st_mtime;
  pst->orfa_st_ctime = st->st_ctime;
}


void convert_statfs_64_to_portable(struct portable_statfs *pst,
				   struct statfs64 *st)
{
  pst->f_type = st->f_type;
  pst->f_bsize = st->f_bsize;
  pst->f_blocks = st->f_blocks;
  pst->f_bfree = st->f_bfree;
  pst->f_bavail = st->f_bavail;
  pst->f_files = st->f_files;
  pst->f_ffree = st->f_ffree;
  pst->f_fsid.__val[0] = st->f_fsid.__val[0];
  pst->f_fsid.__val[1] = st->f_fsid.__val[1];
  pst->f_namelen = st->f_namelen;
  pst->f_spare[0] = st->f_spare[0];
  pst->f_spare[1] = st->f_spare[1];
  pst->f_spare[2] = st->f_spare[2];
  pst->f_spare[3] = st->f_spare[3];
  pst->f_spare[4] = st->f_spare[4];
  pst->f_spare[5] = st->f_spare[5];
}


void convert_dirent_64_to_portable(struct portable_dirent *pd,
				   struct dirent64 *d)
{
  pd->d_ino = d->d_ino;
  pd->d_off = d->d_off;
  pd->d_reclen = d->d_reclen;
  pd->d_type = d->d_type;
  strcpy(pd->d_name, d->d_name);
}


int convert_oflags_from_portable(orfa_oflags_t poflags)
{
  int oflags = 0;

  switch ( poflags & P_O_ACCMODE )
    {
    case P_O_RDONLY: oflags = O_RDONLY; break;
    case P_O_WRONLY: oflags = O_WRONLY; break;
    case P_O_RDWR: oflags = O_RDWR; break;
    }
    
  if ( poflags & P_O_CREAT ) oflags |= O_CREAT;
  if ( poflags & P_O_EXCL ) oflags |= O_EXCL;
  if ( poflags & P_O_NOCTTY ) oflags |= O_NOCTTY;
  if ( poflags & P_O_TRUNC ) oflags |= O_TRUNC;

  if ( poflags & P_O_APPEND ) oflags |= O_APPEND;
  if ( poflags & P_O_NONBLOCK ) oflags |= O_NONBLOCK;
  if ( poflags & P_O_NDELAY ) oflags |= O_NDELAY;
  if ( poflags & P_O_SYNC ) oflags |= O_SYNC;

  if ( poflags & P_O_FSYNC ) oflags |= O_FSYNC;
  if ( poflags & P_O_ASYNC ) oflags |= O_ASYNC;
  if ( poflags & P_O_DSYNC ) oflags |= O_DSYNC;
  if ( poflags & P_O_RSYNC ) oflags |= O_RSYNC;

  if ( poflags & P_O_NOFOLLOW ) oflags |= O_NOFOLLOW;
  if ( poflags & P_O_DIRECTORY ) oflags |= O_DIRECTORY;
  if ( poflags & P_O_LARGEFILE ) oflags |= O_LARGEFILE;
  if ( poflags & P_O_DIRECT ) oflags |= O_DIRECT;

  return oflags;
}


orfa_error_t convert_errno_to_portable(int error)
{
  return (orfa_error_t)error;
}
